const aws = require('aws-sdk');
const awsXRay = require('aws-xray-sdk-core');
const {DEFAULT_AWS_REGION} = require('./constants');

/**
 * Enables x-ray tracing for AWS events (if applicable).
 * @return {Object} - aws-sdk object
 */
const getWrappedAWS = () => {
    if (!process.env._X_AMZN_TRACE_ID) {
        return aws;
    }

    /* istanbul ignore next */
    return awsXRay.captureAWS(aws);
};

const AWS = getWrappedAWS();
AWS.config.update({
    region: process.env.AWS_REGION || DEFAULT_AWS_REGION
});

module.exports = AWS;
